use crate::db::schema::DollProfile;
use sqlx::{pool::PoolConnection, Postgres};

use super::schema::{CreateDollProfile, DollTagsDb};

pub async fn get(
	mut db: DollTagsDb,
	ident: i32,
	microchip_id: &str,
) -> sqlx::Result<Option<DollProfile>> {
	sqlx::query_as!(
		DollProfile,
		r#"
			select * from doll_profiles where id = $1 or microchip_id = $2
		"#,
		ident,
		microchip_id
	)
	.fetch_optional(&mut **db)
	.await
}

pub async fn check_ids(mut db: DollTagsDb, idents: &Vec<i32>) -> sqlx::Result<Vec<i32>> {
	sqlx::query_scalar!(
		"select id from doll_profiles where id in (select * from unnest($1::int[]))",
		idents
	)
	.fetch_all(&mut **db)
	.await
}

pub async fn id_exists(
	db: &mut PoolConnection<Postgres>,
	ident: i32,
	microchip_id: &str,
) -> sqlx::Result<bool> {
	Ok(sqlx::query!(
		"select id from doll_profiles where id = $1 or microchip_id = $2",
		ident,
		microchip_id
	)
	.fetch_optional(&mut **db)
	.await?
	.is_some())
}

pub async fn create(mut db: DollTagsDb, doll: CreateDollProfile<'_>) -> sqlx::Result<()> {
	sqlx::query!(
		r#"
			insert into doll_profiles
			(id, microchip_id, name, pronoun_subject, pronoun_object, pronoun_possessive, handler_name, handler_link, kind, breed, behaviour, description, chassis_type, chassis_id, chassis_color, bound_to_id)
			values ($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16)
		"#,
		doll.id,
		doll.microchip_id,
		doll.name,
		doll.pronoun_subject,
		doll.pronoun_object,
		doll.pronoun_possessive,
		doll.handler_name,
		doll.handler_link,
		doll.kind,
		doll.breed,
		doll.behaviour,
		doll.description,
		doll.chassis_type,
		doll.chassis_id,
		doll.chassis_color,
		doll.bound_to_id,
	).execute(&mut **db).await?;

	Ok(())
}
